#define PIC_PKT_LEN  512 //data length of each read, dont set this too big because ram is limited
#define CAM_ADDR     0
//Color Type
#define CT_GRAYSCALE_2  0x01
#define CT_GRAYSCALE_4  0x02
#define CT_GRAYSCALE_8  0x03
#define CT_COLOR_12     0x05
#define CT_COLOR_16     0x06
#define CT_JPEG         0x07

//Preview Resolution
#define PR_80x60    0x01
#define PR_160x120  0x03

// JPEG Resolution
#define JR_80x64     0x01
#define JR_160x128   0x03
#define JR_320x240   0x05
#define JR_640x480   0x07


const byte camera_address = (CAM_ADDR << 5); // address
unsigned int camera_packet_num;
unsigned int camera_last_packet_len;

void cameraClearRxBuf(){
	while (Serial.available()){
		Serial.read();
	}
}

void cameraSendCmd(char cmd[], int cmd_len){
	for (char i = 0; i < cmd_len; i++) Serial.print(cmd[i]);
}

unsigned int cameraPacketNum(){
	return camera_packet_num;
}

void cameraInit(int color_type, int preview_resolution, int jpeg_resolution){
	char cmd[] = {0xaa,0x0d|camera_address,0x00,0x00,0x00,0x00} ;
	unsigned char resp[6];

	Serial.setTimeout(500);
	while (1){
		cameraSendCmd(cmd,6);

		if(Serial.readBytes((char *)resp, 6) != 6)
			continue;

		if(resp[0] == 0xaa && resp[1] == (0x0e | camera_address) && resp[2] == 0x0d && resp[4] == 0 && resp[5] == 0) {
			if(Serial.readBytes((char *)resp, 6) != 6) continue;
			if(resp[0] == 0xaa && resp[1] == (0x0d | camera_address) && resp[2] == 0 && resp[3] == 0 && resp[4] == 0 && resp[5] == 0) break;
		}
	}

	cmd[1] = 0x0e | camera_address;
	cmd[2] = 0x0d;
	cameraSendCmd(cmd, 6);

	char cmd2[] = { 0xaa, 0x01 | camera_address, 0x00, color_type, preview_resolution, jpeg_resolution};

	Serial.setTimeout(100);

	while (1){
		cameraClearRxBuf();
		cameraSendCmd(cmd2, 6);
		if(Serial.readBytes((char *)resp, 6) != 6) continue;
		if(resp[0] == 0xaa && resp[1] == (0x0e | camera_address) && resp[2] == 0x01 && resp[4] == 0 && resp[5] == 0) break;
	}

	char cmd3[] = { 0xaa, 0x06 | camera_address, 0x08, PIC_PKT_LEN & 0xff, (PIC_PKT_LEN>>8) & 0xff ,0};

	while (1) {
		cameraClearRxBuf();
		cameraSendCmd(cmd3, 6);
		if(Serial.readBytes((char *)resp, 6) != 6) continue;
		if(resp[0] == 0xaa && resp[1] == (0x0e | camera_address) && resp[2] == 0x06 && resp[4] == 0 && resp[5] == 0) break;
	}
}
long cameraGetPicture(){
	char cmd[] = { 0xaa, 0x04 | camera_address, 0x01, 0x00, 0x00, 0x00 };
	unsigned char resp[6];
	unsigned long picTotalLen = 0; // picture length

	while (1){
		cameraClearRxBuf();
		cameraSendCmd(cmd, 6);

		if(Serial.readBytes((char *)resp, 6) != 6) continue;
		if(resp[0] == 0xaa && resp[1] == (0x0e | camera_address) && resp[2] == 0x04 && resp[4] == 0 && resp[5] == 0){
			Serial.setTimeout(1000);
			if(Serial.readBytes((char *)resp, 6) != 6)
				continue;

			if(resp[0] == 0xaa && resp[1] == (0x0a | camera_address) && resp[2] == 0x01){
				picTotalLen = (resp[3]) | (resp[4] << 8) | (resp[5] << 16);
				break;
			}
		}
	}

	camera_packet_num = (picTotalLen) / (PIC_PKT_LEN - 6);
	camera_last_packet_len = PIC_PKT_LEN;

	if((picTotalLen % (PIC_PKT_LEN-6)) != 0){
		camera_packet_num += 1;
		camera_last_packet_len = picTotalLen % (PIC_PKT_LEN - 6) + 6;
	}

	return picTotalLen;
}
long cameraGetPacket(unsigned int i, char* buf){
	char cmd[] = { 0xaa, 0x0e | camera_address, 0x00, 0x00, 0x00, 0x00 };

	Serial.setTimeout(100);

	if(i < camera_packet_num) {
		cmd[4] = i & 0xff;
		cmd[5] = (i >> 8) & 0xff;

		cameraClearRxBuf();
		cameraSendCmd(cmd, 6);

		int pkt_len;

		if(i < (camera_packet_num - 1))
			pkt_len = PIC_PKT_LEN ;
		else
			pkt_len = camera_last_packet_len;

		uint16_t cnt = Serial.readBytes((char *)buf, pkt_len);

		return cnt;
	} else {
		cmd[4] = 0xf0;
		cmd[5] = 0xf0;
		cameraSendCmd(cmd, 6);
	}

	return "";
}